@@ -56,6 +56,7 @@ class ArteTV(QObject):
         self.videos = []
         self.is_loading = False
         self.is_in_category = False
+        self.current_category = "arte+7"
         lang = QLocale.system().name()
         self.locale = QLocale(lang)
         self.downloader = Downloader(self)
@@ -130,6 +131,7 @@ class ArteTV(QObject):
 
         lgg.info('arte tv list updated')
         self.categories = [(k, "") for k in self.categories_videos.keys()]
+        print(self.categories)
         self.categories.insert(0, ["arte+7", ""])
         self.clean_lists()
         self.ui.set_categories_items(self.categories)
@@ -224,7 +226,6 @@ class ArteTV(QObject):
         self.ui.pitch_editor.appendPlainText(item.title)
         self.ui.pitch_editor.setCurrentCharFormat(c_form)
         t = "  %s\n %s %s\n" % (item.duration, item.origin, item.anno)
-        #t = "  %s" % item.duration
         self.ui.pitch_editor.insertPlainText(t)
         if item.realisation:
             rea = ", ".join(item.realisation)
@@ -233,6 +234,10 @@ class ArteTV(QObject):
             music = ", ".join(item.music)
             self.ui.pitch_editor.appendPlainText("  Music: %s" % music)
         desc = item.description
+        if not desc:
+            desc = item.teaser
+            if not desc:
+                desc = "No description."
         self.ui.pitch_editor.appendPlainText('')
         if "</" in desc:
             self.ui.pitch_editor.appendHtml("    %s" % desc)
@@ -264,47 +269,60 @@ class ArteTV(QObject):
             item.url = "https://www.arte.tv" + item.url
         item.player = "https://api.arte.tv/api/player/v2/config/%s/%s" \
                         %(self.lang, item.idx)
-        desc, origin, anno, duration, rea, country = '', '', '', '', '', ''
         page = self.get_page_content(item.url)
-
         if page:
+            if self.current_category == "Séries":
+                if not "RC-" in item.url:
+                    self.get_serial_item(item, page)
+                    return
+
+                else:
+                    self.get_serial_collection(item, page)
+                    return
+
+            parser = ItemParser()
+            parser.feed(page)
+            data = parser.item
+            if data["desc"]:
+                item.description = data["desc"]
             try:
-                content = page.replace('\\"', '"')
-                data = content.split('"content":{"data":[')[1]
-                data = data.split(',"pagination":null}}')[0]
-                donn = "%s%s%s" %('{"data":[', data, '}')
-                donn = donn.replace(r'\\"', "")
-                data = json.loads(donn)
-                meta = data["data"][0]
-            except Exception as why:
-                lgg.info("Read error: %s" % why)
-                #return
-                with open("item.jsn", "w") as outf:
-                    txt = json.dumps(data, sort_keys=False, indent=2, 
-                        separators=(',', ': '), ensure_ascii=False)
-                    outf.write(txt)
-                return
+                if data["teaser"]:
+                    item.teaser = data["teaser"]
+            except KeyError:
+                pass
+            item.duration = data["duration"]
+            item.origin = ", ".join(data["country"])
+            item.anno = data["date"]
+            item.realisation = data["real"]
+            item.actors = data["cast"]
+            print(data["country"], data["real"])
 
-            #item.player = meta["config"]["url"]
-            item.description = meta["shortDescription"]
-            if not "RC" in item.idx:
-                item.duration = self.format_duration(meta.get("duration", 0))
-                credits = meta["credits"]
-                for c in credits:
-                    if c["code"] == "COUNTRY":
-                        item.origin = ", ".join(c["values"])
+    def get_serial_item(self, item, content):
+        lgg.info("Get serial item")
+        try:
+            data = json.loads(content)
+        except Exception as why:
+            lgg.info("json read error: %s" % why)
+            #print(content[:100])
+            return
 
-                    elif c["code"] == "REA":
-                        item.realisation = c["values"]
+        meta = data["data"]["attributes"]["metadata"]
+        item.idx = meta["providerId"]
+        item.title = meta["title"]
+        item.description = meta["description"]
+        item.duration = item.format_duration(meta["duration"]["seconds"])
+        streams, error = self.read_streams(content)
+        if streams:
+            labels = self.sort_streams_by_label(streams)
+            item.streams = self.get_qualities(labels)
 
-                    elif c["code"] == "ACT":
-                        item.actors = c["values"]
+        else:
+            lgg.warning("Streams read error: %s" % error)
 
-                    elif c["code"] == "PRODUCTION_YEAR":
-                        item.anno = c["values"][0]
-
-                    elif c["code"] == "MUS":
-                        item.music = c["values"]
+    def get_serial_collection(self, item, page):
+        parser = SerialParser()
+        parser.feed(page)
+            
 
     def on_item_added(self, idx):
         """Verify the stream's urls for a video.
@@ -579,7 +597,7 @@ class ArteTV(QObject):
         req = urllib.request.Request(url, data=None) # headers=header)
         
         try:
-            content = urllib.request.urlopen(req)
+            content = urllib.request.urlopen(req, timeout=10)
             return str(content.read().decode('utf-8', 'replace'))
         except Exception as why:
             lgg.info('HTTP error (1): %s, %s' % (url, why))
@@ -597,6 +615,8 @@ class ArteTV(QObject):
 
     def change_category(self, name):
         lgg.info("Change category: %s" % name)
+        self.current_category = name
+
         if name == "arte+7":
             self.remove_filter()
             self.is_in_category = False
@@ -747,6 +767,7 @@ class ArteTV(QObject):
             self.ui.refresh_btn.setEnabled(True)
 
     def display_sub_category(self, item):
+        lgg.info("display_sub_category")
         url = "https://api.arte.tv/api/player/v2/config/%s/%s" %(self.lang, item.idx)
         content = self.get_page_content(url)
         if not content:
@@ -780,7 +801,8 @@ class ArteTV(QObject):
             d["title"] = vid
             d["description"] = item.description
             d["duration"] = item.duration
-            d["href"] = "https://api.arte.tv/api/player/v2/config/%s/%s" %(self.lang, idx)
+            d["href"] = "https://api.arte.tv/api/player/v2/config/%s/%s" %(
+                                                            self.lang, idx)
             d["thumbnail_url"] = ""
             videos.append(d)
 
@@ -1208,6 +1230,156 @@ class HtmlParser(HTMLParser):
             self.in_image = False
             self.videos.append(self.item)
 
+class SerialParser(HTMLParser):
+    """Parser general"""
+    def __init__(self):
+        super().__init__()
+        videos = []
+        self.in_item = False
+        self.in_title = False
+        self.in_title1 = False
+        self.in_desc = False
+
+    def handle_starttag(self, tag, attrs):
+        if tag == "div":
+               for attr in attrs:
+                    if attr == ("data-testid", "ts-tsItem"):
+                        self.in_item = True
+                        self.item = {}
+
+        if self.in_item:
+            if tag == "a":
+               for attr in attrs:
+                    if attr[0] == "href":
+                        self.item["href"] = attr[1]
+
+            if tag == "img":
+                for attr in attrs:
+                    if attr[0] == "src":
+                        self.item["src"] = attr[1]
+
+            if tag == "div":
+                for attr in attrs:
+                    if attr == ("class", "ds-1h05j9g"):
+                        self.in_title = True
+
+        if self.in_title:
+            if tag == "h3":
+                self.in_title1 = True
+
+            elif tag == "p":
+                self.in_desc = True
+
+    def handle_data(self, data):
+        if self.in_title1:
+            self.item["title"] = data
+            self.in_title1 = False
+
+        elif self.in_desc:
+            self.item["description"] = data
+            self.in_desc = False
+            self.in_title = False
+            self.in_item = False
+            self.videos.append(self.item)
+
+class ItemParser(HTMLParser):
+    """Parser general"""
+    def __init__(self):
+        super().__init__()
+        self.in_duration = False
+        self.in_desc = False
+        self.in_teaser = False
+        self.in_cast = False
+        self.in_label = False
+        self.in_cast1 = False
+        self.in_actor = False
+        self.in_real = False
+        self.in_real1 = False
+        self.in_orig = False
+        self.in_orig1 = False
+        self.in_date = False
+        self.in_date1 = False
+        self.item = {"desc": "", "cast": [], "real": [],
+                     "country": [], "date": None, "duration": ""}
+
+    def handle_starttag(self, tag, attrs):
+        if tag == "div":
+            for attr in attrs:
+                if attr == ("class", "ds-66uxch"):
+                    self.in_duration = True
+                elif attr == ("class", "ds-t47jm3"):
+                    self.in_desc = True
+                elif attr == ("class", "ds-qcz0i0"):
+                    self.in_teaser = True
+                elif attr == ("class", "ds-1i28n7i"):
+                    self.in_desc = False
+                    self.in_cast = True
+
+        elif tag == "p":
+            for attr in attrs:
+                if attr == ("class", "ds-1rjvccl"):
+                    self.in_label = True
+                elif attr == ("class", "ds-ykt0na"):
+                    if self.in_cast1:
+                        self.in_actor = True
+                    elif self.in_real:
+                        self.in_real1 = True
+                        self.in_actor = False
+                    elif self.in_orig:
+                        self.in_orig1 = True
+                        self.in_real = False
+                        self.in_real1 = False
+                    elif self.in_date:
+                        self.in_orig1 = False
+                        self.in_date1 = True
+
+    def handle_data(self, data):
+        if self.in_duration:
+            self.item["duration"] = data
+            self.in_duration = False
+        elif self.in_label:
+            if data in ("Avec", "Mit"):
+                self.in_cast1 = True
+                self.in_label = False
+            elif data in ("Réalisation", "Regie"):
+                self.in_real = True
+                self.in_cast1 = False
+                self.in_label = False
+            elif data in ("Pays", "Land"):
+                self.in_orig = True
+                self.in_real = False
+                self.in_real1 = False
+                self.in_label = False
+            elif data in ("Année", "Jahr"):
+                self.in_date = True
+                self.in_real1 = False
+                self.in_orig = False
+                self.in_label = False
+            else:
+                self.in_actor = False
+                self.in_real1 = False
+                self.in_orig1 = False
+                self.in_date1 = False
+
+        elif self.in_desc:
+            self.item["desc"] += data
+        elif self.in_teaser:
+            self.item["teaser"] = data
+            self.in_teaser = False
+        elif self.in_actor:
+            self.in_actor = False
+            self.item["cast"].append(data)
+        elif self.in_real1:
+            self.item["real"].append(data)
+            self.in_real1 = False
+        elif self.in_orig1:
+            self.item["country"].append(data)
+            self.in_orig1 = False
+        elif self.in_date1:
+            self.item["date"] = data
+            self.in_date = False
+            self.in_date1 = False
+
 
 class TVItem:
     """Define a video item.
